@@ -3,7 +3,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Lars Gullik Bjnnes
+ * \author Lars Gullik Bjønnes
  * \author Jean-Marc Lasgouttes
  * \author Dekel Tsur
  *
@@ -533,31 +533,35 @@ docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem,
 }
 
 
-void Encodings::initUnicodeMath(Buffer const & buffer, bool clear_sets)
+void Encodings::initUnicodeMath(Buffer const & buffer, bool for_master)
 {
 #ifdef TEX2LYX
 	// The code below is not needed in tex2lyx and requires additional stuff
 	(void)buffer;
-	(void)clear_sets;
+	(void)for_master;
 #else
-	if (clear_sets) {
+	if (for_master) {
 		mathcmd.clear();
 		textcmd.clear();
 		mathsym.clear();
 	}
 
-	// Check master
+	// Check this buffer
 	Inset & inset = buffer.inset();
 	InsetIterator it = inset_iterator_begin(inset);
 	InsetIterator const end = inset_iterator_end(inset);
 	for (; it != end; ++it)
 		it->initUnicodeMath();
 
+	if (!for_master)
+		return;
+
 	// Check children
-	std::vector<Buffer *> clist = buffer.getChildren();
-	for (vector<Buffer *>::const_iterator cit = clist.begin();
-	     cit != clist.end(); ++cit) 
-		initUnicodeMath(**cit, false);
+	ListOfBuffers blist = buffer.getDescendents();
+	ListOfBuffers::const_iterator bit = blist.begin();
+	ListOfBuffers::const_iterator const bend = blist.end();
+	for (; bit != bend; ++bit)
+		initUnicodeMath(**bit, false);
 #endif
 }
 
@@ -824,7 +828,7 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 	// Now read the encodings
 	enum {
 		et_encoding = 1,
-		et_end,
+		et_end
 	};
 
 	LexerKeyword encodingtags[] = {
