@@ -4,7 +4,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Lars Gullik Bjnnes
+ * \author Lars Gullik Bjønnes
  * \author Angus Leeming
  * \author Dekel Tsur
  *
@@ -39,16 +39,20 @@ private:
 		///
 		Error () : error_in_line(0) {}
 		///
-		Error(int line, docstring const & desc, docstring const & text)
+		Error(int line, docstring const & desc, docstring const & text,
+		      std::string const & fname)
 			: error_in_line(line),
 			  error_desc(desc),
-			  error_text(text) {}
+			  error_text(text),
+			  child_name(fname) {}
 		/// what line in the TeX file the error occured in
 		int error_in_line;
 		/// The kind of error
 		docstring error_desc;
 		/// The line/cmd that caused the error.
 		docstring error_text;
+		/// The name of the child where error occurred, empty otherwise.
+		std::string child_name;
 	};
 public:
 	///
@@ -59,17 +63,18 @@ public:
 	Errors::const_iterator end() const { return errors.end(); }
 	///
 	void insertError(int line, docstring const & error_desc,
-			 docstring const & error_text);
+			 docstring const & error_text,
+			 std::string const & child_name = empty_string());
 private:
 	///
 	Errors errors;
 };
 
 
-class Aux_Info {
+class AuxInfo {
 public:
 	///
-	Aux_Info() {}
+	AuxInfo() {}
 	///
 	support::FileName aux_file;
 	///
@@ -82,8 +87,8 @@ public:
 
 
 ///
-bool operator==(Aux_Info const &, Aux_Info const &);
-bool operator!=(Aux_Info const &, Aux_Info const &);
+bool operator==(AuxInfo const &, AuxInfo const &);
+bool operator!=(AuxInfo const &, AuxInfo const &);
 
 
 /**
@@ -146,7 +151,8 @@ public:
 	   path = name of the files original path.
 	*/
 	LaTeX(std::string const & cmd, OutputParams const &,
-	      support::FileName const & file);
+	      support::FileName const & file,
+	      std::string const & path = empty_string());
 
 	/// runs LaTeX several times
 	int run(TeXErrors &);
@@ -176,20 +182,25 @@ private:
 				 std::string const &, std::string const &);
 
 	///
-	std::vector<Aux_Info> const scanAuxFiles(support::FileName const &);
+	std::vector<AuxInfo> const scanAuxFiles(support::FileName const &);
 
 	///
-	Aux_Info const scanAuxFile(support::FileName const &);
+	AuxInfo const scanAuxFile(support::FileName const &);
 
 	///
-	void scanAuxFile(support::FileName const &, Aux_Info &);
+	void scanAuxFile(support::FileName const &, AuxInfo &);
 
 	///
 	void updateBibtexDependencies(DepTable &,
-				      std::vector<Aux_Info> const &);
+				      std::vector<AuxInfo> const &,
+				      bool biber);
 
 	///
-	bool runBibTeX(std::vector<Aux_Info> const &, OutputParams const &);
+	void scanBlgFile(DepTable & head);
+
+	///
+	bool runBibTeX(std::vector<AuxInfo> const &,
+		       OutputParams const &, bool biber);
 
 	///
 	void deleteFilesOnError() const;
@@ -200,6 +211,9 @@ private:
 	///
 	support::FileName file;
 
+	///
+	std::string path;
+
 	/// used by scanLogFile
 	int num_errors;
 
