@@ -3,7 +3,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Lars Gullik Bjnnes
+ * \author Lars Gullik Bjønnes
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -31,7 +31,7 @@ public:
 	docstring const get(std::string const & msg) const;
 private:
 	class Pimpl;
-	boost::scoped_ptr<Pimpl> pimpl_;
+	::boost::scoped_ptr<Pimpl> pimpl_;
 };
 
 
