@@ -4,7 +4,7 @@
  * Licence details can be found in the file COPYING.
  *
  * \author Angus Leeming
- * \author Jrgen Spitzmller
+ * \author Jürgen Spitzmüller
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -15,14 +15,15 @@
 #include "qt_helpers.h"
 
 #include "Buffer.h"
-#include "Converter.h"
+#include "BufferParams.h"
 #include "Format.h"
 #include "FuncRequest.h"
-#include "LyXRC.h"
 
 #include "support/qstring_helpers.h"
 #include "support/filetools.h"
 
+#include <algorithm>
+
 #include <QLineEdit>
 #include <QListWidget>
 #include <QPushButton>
@@ -35,7 +36,7 @@ namespace frontend {
 
 
 GuiSendTo::GuiSendTo(GuiView & lv)
-	: GuiDialog(lv, "sendto", qt_("Send Document to Command"))
+	: GuiDialog(lv, "sendto", qt_("Export or Send Document"))
 {
 	setupUi(this);
 
@@ -51,7 +52,7 @@ GuiSendTo::GuiSendTo(GuiView & lv)
 		this, SLOT(changed_adaptor()));
 	connect(formatLW, SIGNAL(itemSelectionChanged()),
 		this, SLOT(changed_adaptor()));
-	connect(commandCO, SIGNAL(textChanged(QString)),
+	connect(commandCO, SIGNAL(editTextChanged(QString)),
 		this, SLOT(changed_adaptor()));
 
 	bc().setPolicy(ButtonPolicy::OkApplyCancelPolicy);
@@ -66,10 +67,17 @@ void GuiSendTo::changed_adaptor()
 	changed();
 }
 
+namespace {
+bool formatSorter(Format const * lhs, Format const * rhs) {
+	return lhs->prettyname() < rhs->prettyname();
+}
+} // end namespace
 
 void GuiSendTo::updateContents()
 {
-	all_formats_ = allFormats();
+	all_formats_ = buffer().params().exportableFormats(false);
+	
+	sort(all_formats_.begin(), all_formats_.end(), formatSorter);
 
 	// Save the current selection if any
 	Format const * current_format = 0;
@@ -133,15 +141,13 @@ bool GuiSendTo::isValid()
 		return false;
 
 	return (formatLW->selectedItems().size() > 0
-		&& formatLW->count() != 0
-		&& !commandCO->currentText().isEmpty());
+		&& formatLW->count() != 0);
 }
 
 
 bool GuiSendTo::initialiseParams(string const &)
 {
 	format_ = 0;
-	command_ = toqstr(lyxrc.custom_export_command);
 	paramsToDialog(format_, command_);
 	return true;
 }
@@ -158,66 +164,23 @@ void GuiSendTo::paramsToDialog(Format const * /*format*/, QString const & comman
 
 void GuiSendTo::dispatchParams()
 {
-	if (command_.isEmpty() || !format_ || format_->name().empty())
+	if (!format_ || format_->name().empty())
 		return;
 
-	string const data = format_->name() + " " + fromqstr(command_);
-	dispatch(FuncRequest(getLfun(), data));
+	string data = format_->name();
+	if (!command_.isEmpty())
+		data += " " + fromqstr(command_);
+
+	FuncCode const lfun = command_.isEmpty() ?
+		LFUN_BUFFER_EXPORT : getLfun();
+
+	dispatch(FuncRequest(lfun, data));
 }
 
-// FIXME: Move to Converters?
-vector<Format const *> GuiSendTo::allFormats() const
-{
-	// What formats can we output natively?
-	vector<string> exports;
-	exports.push_back("lyx");
-	exports.push_back("text");
-
-	if (buffer().isLatex()) {
-		exports.push_back("latex");
-		exports.push_back("pdflatex");
-	}
-	else if (buffer().isDocBook())
-		exports.push_back("docbook");
-	else if (buffer().isLiterate())
-		exports.push_back("literate");
-
-	// Loop over these native formats and ascertain what formats we
-	// can convert to
-	vector<Format const *> to;
-
-	vector<string>::const_iterator ex_it  = exports.begin();
-	vector<string>::const_iterator ex_end = exports.end();
-	for (; ex_it != ex_end; ++ex_it) {
-		// Start off with the native export format.
-		// "formats" is LyX's list of recognised formats
-		to.push_back(formats.getFormat(*ex_it));
-
-		Formats::const_iterator fo_it  = formats.begin();
-		Formats::const_iterator fo_end = formats.end();
-		for (; fo_it != fo_end; ++fo_it) {
-			// we need to hide the default graphic export formats
-			// from the external menu, because we need them only
-			// for the internal lyx-view and external latex run
-			string const name = fo_it->name();
-			if (name != "eps" && name != "xpm" && name != "png" &&
-			    theConverters().isReachable(*ex_it, name))
-				to.push_back(&(*fo_it));
-		}
-	}
-
-	// Remove repeated formats.
-	sort(to.begin(), to.end());
-	to.erase(unique(to.begin(), to.end()), to.end());
-
-	return to;
-}
-
-
 Dialog * createGuiSendTo(GuiView & lv) { return new GuiSendTo(lv); }
 
 
 } // namespace frontend
 } // namespace lyx
 
-#include "GuiSendto_moc.cpp"
+#include "moc_GuiSendto.cpp"
