@@ -600,15 +600,11 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 		tmp->markDepClean(masterBuffer->temppath());
 
 		// Don't assume the child's format is latex
-		string const inc_format = tmp->bufferFormat();
+		string const inc_format = tmp->params().bufferFormat();
 		FileName const tmpwritefile(changeExtension(writefile.absFileName(),
 			formats.extension(inc_format)));
 
 		// FIXME: handle non existing files
-		// FIXME: Second argument is irrelevant!
-		// since only_body is true, makeLaTeXFile will not look at second
-		// argument. Should we set it to string(), or should makeLaTeXFile
-		// make use of it somehow? (JMarc 20031002)
 		// The included file might be written in a different encoding
 		// and language.
 		Encoding const * const oldEnc = runparams.encoding;
@@ -621,13 +617,12 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 		runparams.master_language = buffer().params().language;
 		runparams.par_begin = 0;
 		runparams.par_end = tmp->paragraphs().size();
-		if (!tmp->makeLaTeXFile(tmpwritefile, masterFileName(buffer()).
-				onlyPath().absFileName(), runparams, false)) {
+		if (!tmp->makeLaTeXFile(tmpwritefile, runparams, false)) {
 			docstring msg = bformat(_("Included file `%1$s' "
 					"was not exported correctly.\nWarning: "
 					"LaTeX export is probably incomplete."),
 					included_file.displayName());
-			ErrorList & el = tmp->errorList("Export");
+			ErrorList const & el = tmp->errorList("Export");
 			if (!el.empty())
 				msg = bformat(from_ascii("%1$s\n\n%2$s\n\n%3$s"),
 						msg, el.begin()->error,
@@ -661,7 +656,8 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 		// In this case, it's not a LyX file, so we copy the file
 		// to the temp dir, so that .aux files etc. are not created
 		// in the original dir. Files included by this file will be
-		// found via input@path, see ../Buffer.cpp.
+		// found via the environment variable TEXINPUTS, which may be
+		// set in preferences and by default includes the original dir.
 		unsigned long const checksum_in  = included_file.checksum();
 		unsigned long const checksum_out = writefile.checksum();
 
@@ -734,7 +730,7 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 }
 
 
-docstring InsetInclude::xhtml(XHTMLStream & xs, OutputParams const &rp) const
+docstring InsetInclude::xhtml(XHTMLStream & xs, OutputParams const & rp) const
 {
 	if (rp.inComment)
 		 return docstring();
@@ -778,7 +774,23 @@ docstring InsetInclude::xhtml(XHTMLStream & xs, OutputParams const &rp) const
 	Buffer const * const ibuf = loadIfNeeded();
 	if (!ibuf)
 		return docstring();
-	ibuf->writeLyXHTMLSource(xs.os(), rp, true);
+
+	// are we generating only some paragraphs, or all of them?
+	bool const all_pars = !rp.dryrun || 
+			(rp.par_begin == 0 && 
+			 rp.par_end == (int)buffer().text().paragraphs().size());
+	
+	OutputParams op = rp;
+	if (all_pars) {
+		op.par_begin = 0;
+		op.par_end = 0;
+		ibuf->writeLyXHTMLSource(xs.os(), op, true);
+	} else
+		xs << XHTMLStream::ESCAPE_NONE 
+		   << "<!-- Included file: " 
+		   << from_utf8(included_file.absFileName()) 
+		   << XHTMLStream::ESCAPE_NONE 
+			 << " -->";
 	return docstring();
 }
 
