@@ -40,9 +40,13 @@ public:
 
 	/** Start child process.
 	 *  The string "what" contains a commandline with arguments separated
-	 *  by spaces.
+	 *  by spaces. The string "path" contains the path to be prepended to
+	 *  the TEXINPUTS environment variable. Unset "process_events" in
+	 *  case UI should be blocked while processing the external command.
 	 */
-	int startscript(Starttype how, std::string const & what);
+	int startscript(Starttype how, std::string const & what,
+			std::string const & path = empty_string(),
+			bool process_events = false);
 };
 
 } // namespace support
