@@ -13,6 +13,7 @@
 #include "support/docstream.h"
 #include "support/unicode.h"
 
+#include <algorithm>
 #include <cerrno>
 #include <cstdio>
 #include <cstring>
@@ -23,10 +24,12 @@ using namespace std;
 
 using lyx::ucs4_codeset;
 
+
 #if defined(_MSC_VER) && (_MSC_VER >= 1600)
 std::locale::id numpunct<lyx::char_type>::id;
 #endif
 
+
 namespace {
 
 // We use C IO throughout this file, because the facets might be used with
@@ -306,9 +309,9 @@ const char * iconv_codecvt_facet_exception::what() const throw()
 }
 
 
-ifdocstream::ifdocstream(string const & encoding) : base()
+ifdocstream::ifdocstream() : base()
 {
-	setEncoding(*this, encoding, in);
+	setEncoding(*this, "UTF-8", in);
 }
 
 
@@ -399,6 +402,139 @@ odocstream & operator<<(odocstream & os, char c)
 }
 #endif
 
+
+void otexstream::put(char_type const & c)
+{
+	if (protectspace_) {
+		if (!canbreakline_ && c == ' ')
+			os_ << "{}";
+		protectspace_ = false;
+	}
+	os_.put(c);
+	lastchar_ = c;
+	if (c == '\n') {
+		texrow_.newline();
+		canbreakline_ = false;
+	} else
+		canbreakline_ = true;
+}
+
+
+BreakLine breakln;
+SafeBreakLine safebreakln;
+
+
+otexstream & operator<<(otexstream & ots, BreakLine)
+{
+	if (ots.canBreakLine()) {
+		ots.os().put('\n');
+		ots.lastChar('\n');
+		ots.canBreakLine(false);
+		ots.texrow().newline();
+	}
+	ots.protectSpace(false);
+	return ots;
+}
+
+
+otexstream & operator<<(otexstream & ots, SafeBreakLine)
+{
+	if (ots.canBreakLine()) {
+		ots.os() << "%\n";
+		ots.lastChar('\n');
+		ots.canBreakLine(false);
+		ots.texrow().newline();
+	}
+	ots.protectSpace(false);
+	return ots;
+}
+
+
+otexstream & operator<<(otexstream & ots, odocstream_manip pf)
+{
+	ots.os() << pf;
+	if (pf == static_cast<odocstream_manip>(endl)) {
+		ots.lastChar('\n');
+		ots.texrow().newline();
+	}
+	return ots;
+}
+
+
+otexstream & operator<<(otexstream & ots, docstring const & s)
+{
+	size_t const len = s.length();
+
+	// Check whether there's something to output
+	if (len == 0)
+		return ots;
+
+	if (ots.protectSpace()) {
+		if (!ots.canBreakLine() && s[0] == ' ')
+			ots.os() << "{}";
+		ots.protectSpace(false);
+	}
+	ots.os() << s;
+	ots.lastChar(s[len - 1]);
+	ots.texrow().newlines(count(s.begin(), s.end(), '\n'));
+	ots.canBreakLine(s[len - 1] != '\n');
+	return ots;
+}
+
+
+otexstream & operator<<(otexstream & ots, char const * s)
+{
+	size_t const len = strlen(s);
+
+	// Check whether there's something to output
+	if (len == 0)
+		return ots;
+
+	if (ots.protectSpace()) {
+		if (!ots.canBreakLine() && s[0] == ' ')
+			ots.os() << "{}";
+		ots.protectSpace(false);
+	}
+	ots.os() << s;
+	ots.lastChar(s[len - 1]);
+	ots.texrow().newlines(count(s, s + len, '\n'));
+	ots.canBreakLine(s[len - 1] != '\n');
+	return ots;
+}
+
+
+otexstream & operator<<(otexstream & ots, char c)
+{
+	if (ots.protectSpace()) {
+		if (!ots.canBreakLine() && c == ' ')
+			ots.os() << "{}";
+		ots.protectSpace(false);
+	}
+	ots.os() << c;
+	ots.lastChar(c);
+	if (c == '\n')
+		ots.texrow().newline();
+	ots.canBreakLine(c != '\n');
+	return ots;
+}
+
+
+template <typename Type>
+otexstream & operator<<(otexstream & ots, Type value)
+{
+	ots.os() << value;
+	ots.lastChar(0);
+	ots.canBreakLine(true);
+	ots.protectSpace(false);
+	return ots;
+}
+
+template otexstream & operator<< <SetEnc>(otexstream & os, SetEnc);
+template otexstream & operator<< <double>(otexstream &, double);
+template otexstream & operator<< <int>(otexstream &, int);
+template otexstream & operator<< <unsigned int>(otexstream &, unsigned int);
+template otexstream & operator<< <unsigned long>(otexstream &, unsigned long);
+
 }
 
 #if ! defined(USE_WCHAR_T) && defined(__GNUC__)
