@@ -32,9 +32,20 @@ enum path_case {
 	CASE_ADJUSTED
 };
 
+enum file_access {
+	EXISTING,
+	CREATE
+};
+
 /// Do some work just once.
 void init(int argc, char * argv[]);
 
+/// Returns the i-th program argument in utf8 encoding.
+std::string utf8_argv(int i);
+
+/// Removes from the internal copy \p num program arguments starting from \p i.
+void remove_internal_args(int i, int num);
+
 /// Returns the name of the NULL device (/dev/null, null).
 std::string const & nulldev();
 
@@ -44,6 +55,9 @@ std::string current_root();
 ///
 shell_type shell();
 
+/// Returns no. of minutes allowed for a command to complete.
+int timeout_min();
+
 /// Name of the python interpreter
 std::string const python();
 
@@ -64,6 +78,15 @@ std::string external_path(std::string const & p);
 /// \p p and the return value are encoded in utf8.
 std::string internal_path(std::string const & p);
 
+/// Converts a host OS style path to a unicode safe unix style.
+/// On Windows, this is achieved by using the short form of the path,
+/// which can be safely passed to standard I/O functions expecting narrow
+/// char paths even when the path contains non-ascii chars.
+/// As the short form is only available for existing files, if the file is
+/// to be accessed for writing, \param how should be set to CREATE.
+/// \p p and the return value are encoded in utf8.
+std::string safe_internal_path(std::string const & p, file_access how = EXISTING);
+
 /// Converts a unix style path list to host OS style.
 /// \p p and the return value are encoded in utf8.
 std::string external_path_list(std::string const & p);
@@ -81,6 +104,12 @@ std::string internal_path_list(std::string const & p);
  */
 std::string latex_path(std::string const & p);
 
+/**
+ * Converts a platform style path list into a form suitable for the TeX engine.
+ * \p p is encoded in utf8.
+ */
+std::string latex_path_list(std::string const & p);
+
 /// Checks if the format string is suitable on the OS
 bool is_valid_strftime(std::string const & p);
 
@@ -89,10 +118,15 @@ bool is_valid_strftime(std::string const & p);
  */
 char const * popen_read_mode();
 
-/** The character used to separate paths returned by the
- *  PATH environment variable.
+enum path_type {
+	PLATFORM,
+	TEXENGINE
+};
+
+/** The character used to separate paths for platform environment variables
+ *  (such as PATH) or for the TeX engine.
  */
-char path_separator();
+char path_separator(path_type type = PLATFORM);
 
 /** If @c use_windows_paths is true, LyX will output Windows-style paths to
  *  latex files rather than posix ones. Obviously, this option is used only
@@ -110,18 +144,18 @@ enum auto_open_mode {
  *  \param mode can be opened in VIEW or EDIT mode
  *  \returns whether or not the format can be opened according to \p mode
  */
-bool canAutoOpenFile(std::string const & ext, auto_open_mode const mode = VIEW);
+bool canAutoOpenFile(std::string const & ext, auto_open_mode const mode);
 
 /** View or edit a file with the default viewer or editor.
  *  \param filename file to open (encoded in utf8)
  *  \param mode open in VIEW or EDIT mode
  *  \returns whether or not the file is viewed (or edited) successfully.
  */
-bool autoOpenFile(std::string const & filename, auto_open_mode const mode = VIEW);
+bool autoOpenFile(std::string const & filename, auto_open_mode const mode,
+		  std::string const & path = empty_string());
 
 /** Resolves a path such that it does not contain '.', '..', or symbolic links.
-  * \warning the path must already be in the filesystem encoding.
-  * \returns the resolved path in utf8 encoding.
+  * \p path and the return value are encoded in utf8.
   */
 std::string real_path(std::string const & path);
 
