@@ -3,7 +3,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Andr Pnitz
+ * \author André Pönitz
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -17,6 +17,9 @@
 #include "Context.h"
 #include "Encoding.h"
 #include "Layout.h"
+#include "LayoutFile.h"
+#include "LayoutModuleList.h"
+#include "ModuleList.h"
 #include "TextClass.h"
 
 #include "support/convert.h"
@@ -25,8 +28,10 @@
 #include "support/filetools.h"
 #include "support/lassert.h"
 #include "support/lstrings.h"
+#include "support/Messages.h"
 #include "support/os.h"
 #include "support/Package.h"
+#include "support/Systemcall.h"
 
 #include <cstdlib>
 #include <iostream>
@@ -41,6 +46,48 @@ using namespace lyx::support::os;
 
 namespace lyx {
 
+namespace frontend {
+namespace Alert {
+	void warning(docstring const & title, docstring const & message,
+				 bool const &)
+	{
+		LYXERR0(title);
+		LYXERR0(message);
+	}
+}
+}
+
+
+// Dummy texrow support
+void TexRow::newline()
+{}
+
+
+void TexRow::newlines(int)
+{}
+
+
+// Dummy LyXRC support
+class LyXRC {
+public:
+	string icon_set;
+} lyxrc;
+
+
+// Dummy translation support
+Messages messages_;
+Messages const & getMessages(std::string const &)
+{
+	return messages_;
+}
+
+
+Messages const & getGuiMessages()
+{
+	return messages_;
+}
+
+
 // Keep the linker happy on Windows
 void lyx_exit(int)
 {}
@@ -107,13 +154,18 @@ string active_environment()
 }
 
 
+TeX2LyXDocClass textclass;
 CommandMap known_commands;
 CommandMap known_environments;
 CommandMap known_math_environments;
+FullCommandMap possible_textclass_commands;
+FullEnvironmentMap possible_textclass_environments;
+
+/// used modules
+LayoutModuleList used_modules;
 
 
-void add_known_command(string const & command, string const & o1,
-		       bool o2)
+void convertArgs(string const & o1, bool o2, vector<ArgumentType> & arguments)
 {
 	// We have to handle the following cases:
 	// definition                      o1    o2    invocation result
@@ -123,7 +175,6 @@ void add_known_command(string const & command, string const & o1,
 	// \newcommand{\foo}[1][]{bar #1}  "[1]" true  \foo[x]    bar x
 	// \newcommand{\foo}[1][x]{bar #1} "[1]" true  \foo[x]    bar x
 	unsigned int nargs = 0;
-	vector<ArgumentType> arguments;
 	string const opt1 = rtrim(ltrim(o1, "["), "]");
 	if (isStrUnsignedInt(opt1)) {
 		// The command has arguments
@@ -136,11 +187,161 @@ void add_known_command(string const & command, string const & o1,
 	}
 	for (unsigned int i = 0; i < nargs; ++i)
 		arguments.push_back(required);
+}
+
+
+void add_known_command(string const & command, string const & o1,
+                       bool o2, docstring const & definition)
+{
+	vector<ArgumentType> arguments;
+	convertArgs(o1, o2, arguments);
 	known_commands[command] = arguments;
+	if (!definition.empty())
+		possible_textclass_commands[command] =
+			FullCommand(arguments, definition);
+}
+
+
+void add_known_environment(string const & environment, string const & o1,
+                           bool o2, docstring const & beg, docstring const &end)
+{
+	vector<ArgumentType> arguments;
+	convertArgs(o1, o2, arguments);
+	known_environments[environment] = arguments;
+	if (!beg.empty() || ! end.empty())
+		possible_textclass_environments[environment] =
+			FullEnvironment(arguments, beg, end);
+}
+
+
+Layout const * findLayoutWithoutModule(TextClass const & textclass,
+                                       string const & name, bool command)
+{
+	DocumentClass::const_iterator it = textclass.begin();
+	DocumentClass::const_iterator en = textclass.end();
+	for (; it != en; ++it) {
+		if (it->latexname() == name &&
+		    ((command && it->isCommand()) || (!command && it->isEnvironment())))
+			return &*it;
+	}
+	return 0;
+}
+
+
+InsetLayout const * findInsetLayoutWithoutModule(TextClass const & textclass,
+                                                 string const & name, bool command)
+{
+	DocumentClass::InsetLayouts::const_iterator it = textclass.insetLayouts().begin();
+	DocumentClass::InsetLayouts::const_iterator en = textclass.insetLayouts().end();
+	for (; it != en; ++it) {
+		if (it->second.latexname() == name &&
+		    ((command && it->second.latextype() == InsetLayout::COMMAND) ||
+		     (!command && it->second.latextype() == InsetLayout::ENVIRONMENT)))
+			return &(it->second);
+	}
+	return 0;
+}
+
+
+bool checkModule(string const & name, bool command)
+{
+	// Cache to avoid slowdown by repated searches
+	static set<string> failed[2];
+
+	// Only add the module if the command was actually defined in the LyX preamble
+	if (command) {
+		if (possible_textclass_commands.find('\\' + name) == possible_textclass_commands.end())
+			return false;
+	} else {
+		if (possible_textclass_environments.find(name) == possible_textclass_environments.end())
+			return false;
+	}
+	if (failed[command].find(name) != failed[command].end())
+		return false;
+
+	// Create list of dummy document classes if not already done.
+	// This is needed since a module cannot be read on its own, only as
+	// part of a document class.
+	LayoutFile const & baseClass = LayoutFileList::get()[textclass.name()];
+	typedef map<string, DocumentClass *> ModuleMap;
+	static ModuleMap modules;
+	static bool init = true;
+	if (init) {
+		baseClass.load();
+		DocumentClassBundle & bundle = DocumentClassBundle::get();
+		LyXModuleList::const_iterator const end = theModuleList.end();
+		LyXModuleList::const_iterator it = theModuleList.begin();
+		for (; it != end; it++) {
+			string const module = it->getID();
+			LayoutModuleList m;
+			// FIXME this excludes all modules that depend on another one
+			if (!m.moduleCanBeAdded(module, &baseClass))
+				continue;
+			m.push_back(module);
+			modules[module] = &bundle.makeDocumentClass(baseClass, m);
+		}
+		init = false;
+	}
+
+	// Try to find a module that defines the command.
+	// Only add it if the definition can be found in the preamble of the
+	// style that corresponds to the command. This is a heuristic and
+	// different from the way how we parse the builtin commands of the
+	// text class (in that case we only compare the name), but it is
+	// needed since it is not unlikely that two different modules define a
+	// command with the same name.
+	ModuleMap::iterator const end = modules.end();
+	for (ModuleMap::iterator it = modules.begin(); it != end; it++) {
+		string const module = it->first;
+		if (!used_modules.moduleCanBeAdded(module, &baseClass))
+			continue;
+		if (findLayoutWithoutModule(textclass, name, command))
+			continue;
+		if (findInsetLayoutWithoutModule(textclass, name, command))
+			continue;
+		DocumentClass const * c = it->second;
+		Layout const * layout = findLayoutWithoutModule(*c, name, command);
+		InsetLayout const * insetlayout = layout ? 0 :
+			findInsetLayoutWithoutModule(*c, name, command);
+		docstring preamble;
+		if (layout)
+			preamble = layout->preamble();
+		else if (insetlayout)
+			preamble = insetlayout->preamble();
+		if (preamble.empty())
+			continue;
+		bool add = false;
+		if (command) {
+			FullCommand const & cmd =
+				possible_textclass_commands['\\' + name];
+			if (preamble.find(cmd.def) != docstring::npos)
+				add = true;
+		} else {
+			FullEnvironment const & env =
+				possible_textclass_environments[name];
+			if (preamble.find(env.beg) != docstring::npos &&
+			    preamble.find(env.end) != docstring::npos)
+				add = true;
+		}
+		if (add) {
+			FileName layout_file = libFileSearch("layouts", module, "module");
+			if (textclass.read(layout_file, TextClass::MODULE)) {
+				used_modules.push_back(module);
+				// speed up further searches:
+				// the module does not need to be checked anymore.
+				modules.erase(it);
+				return true;
+			}
+		}
+	}
+	failed[command].insert(name);
+	return false;
 }
 
 
 bool noweb_mode = false;
+bool pdflatex = false;
+bool roundtrip = false;
 
 
 namespace {
@@ -162,6 +363,8 @@ void read_command(Parser & p, string command, CommandMap & commands)
 			string const arg = p.getArg('{', '}');
 			if (arg == "translate")
 				arguments.push_back(required);
+			else if (arg == "item")
+				arguments.push_back(item);
 			else
 				arguments.push_back(verbatim);
 		} else {
@@ -259,7 +462,8 @@ int parse_help(string const &, string const &)
 		"\t-f                 Force overwrite of .lyx files.\n"
 		"\t-help              Print this message and quit.\n"
 		"\t-n                 translate a noweb (aka literate programming) file.\n"
-		"\t-s syntaxfile      read additional syntax file.\n" 
+		"\t-roundtrip         re-export created .lyx file infile.lyx.lyx to infile.lyx.tex.\n"
+		"\t-s syntaxfile      read additional syntax file.\n"
 		"\t-sysdir dir        Set system directory to DIR.\n"
 		"\t-userdir DIR       Set user directory to DIR."
 	     << endl;
@@ -340,6 +544,13 @@ int parse_noweb(string const &, string const &)
 }
 
 
+int parse_roundtrip(string const &, string const &)
+{
+	roundtrip = true;
+	return 0;
+}
+
+
 void easyParse(int & argc, char * argv[])
 {
 	map<string, cmd_helper> cmdmap;
@@ -353,6 +564,7 @@ void easyParse(int & argc, char * argv[])
 	cmdmap["-n"] = parse_noweb;
 	cmdmap["-sysdir"] = parse_sysdir;
 	cmdmap["-userdir"] = parse_userdir;
+	cmdmap["-roundtrip"] = parse_roundtrip;
 
 	for (int i = 1; i < argc; ++i) {
 		map<string, cmd_helper>::const_iterator it
@@ -412,19 +624,30 @@ namespace {
  *  You must ensure that \p parentFilePath is properly set before calling
  *  this function!
  */
-void tex2lyx(idocstream & is, ostream & os, string const & encoding)
+void tex2lyx(idocstream & is, ostream & os, string encoding)
 {
+	// Set a sensible default encoding.
+	// This is used until an encoding command is found.
+	// For child documents use the encoding of the master, else latin1,
+	// since latin1 does not cause an iconv error if the actual encoding
+	// is different (bug 7509).
+	if (encoding.empty()) {
+		if (h_inputencoding == "auto")
+			encoding = "latin1";
+		else
+			encoding = h_inputencoding;
+	}
+
 	Parser p(is);
-	if (!encoding.empty())
-		p.setEncoding(encoding);
+	p.setEncoding(encoding);
 	//p.dump();
 
-	stringstream ss;
-	TeX2LyXDocClass textclass;
-	parse_preamble(p, ss, documentclass, textclass);
+	ostringstream ps;
+	parse_preamble(p, ps, documentclass, textclass);
 
 	active_environments.push_back("document");
 	Context context(true, textclass);
+	stringstream ss;
 	parse_text(p, ss, FLAG_END, true, context);
 	if (Context::empty)
 		// Empty document body. LyX needs at least one paragraph.
@@ -432,6 +655,19 @@ void tex2lyx(idocstream & is, ostream & os, string const & encoding)
 	context.check_end_layout(ss);
 	ss << "\n\\end_body\n\\end_document\n";
 	active_environments.pop_back();
+
+	// We know the used modules only after parsing the full text
+	ostringstream ms;
+	if (!used_modules.empty()) {
+		ms << "\\begin_modules\n";
+		LayoutModuleList::const_iterator const end = used_modules.end();
+		LayoutModuleList::const_iterator it = used_modules.begin();
+		for (; it != end; it++)
+			ms << *it << '\n';
+		ms << "\\end_modules\n";
+	}
+	os << subst(ps.str(), modules_placeholder, ms.str());
+
 	ss.seekg(0);
 	os << ss.str();
 #ifdef TEST_PARSER
@@ -457,7 +693,7 @@ bool tex2lyx(FileName const & infilename, ostream & os, string const & encoding)
 		return false;
 	}
 	string const oldParentFilePath = parentFilePath;
-	parentFilePath = onlyPath(infilename.absFilename());
+	parentFilePath = onlyPath(infilename.absFileName());
 	tex2lyx(is, os, encoding);
 	parentFilePath = oldParentFilePath;
 	return true;
@@ -494,6 +730,29 @@ bool tex2lyx(string const & infilename, FileName const & outfilename,
 	return tex2lyx(FileName(infilename), os, encoding);
 }
 
+
+bool tex2tex(string const & infilename, FileName const & outfilename,
+             string const & encoding)
+{
+	if (!tex2lyx(infilename, outfilename, encoding))
+		return false;
+	string command = quoteName(package().lyx_binary().toFilesystemEncoding());
+	if (overwrite_files)
+		command += " -f main";
+	else
+		command += " -f none";
+	if (pdflatex)
+		command += " -e pdflatex ";
+	else
+		command += " -e latex ";
+	command += quoteName(outfilename.toFilesystemEncoding());
+	Systemcall one;
+	if (one.startscript(Systemcall::Wait, command) == 0)
+		return true;
+	cerr << "Error: Running '" << command << "' failed." << endl;
+	return false;
+}
+
 } // namespace lyx
 
 
@@ -501,6 +760,8 @@ int main(int argc, char * argv[])
 {
 	using namespace lyx;
 
+	//setlocale(LC_CTYPE, "");
+
 	lyxerr.setStream(cerr);
 
 	os::init(argc, argv);
@@ -518,19 +779,29 @@ int main(int argc, char * argv[])
 		cerr << to_utf8(message.title_) << ":\n"
 		     << to_utf8(message.details_) << endl;
 		if (message.type_ == ErrorException)
-			exit(1);
+			return EXIT_FAILURE;
 	}
 
 	// Now every known option is parsed. Look for input and output
 	// file name (the latter is optional).
 	string infilename = internal_path(os::utf8_argv(1));
-	infilename = makeAbsPath(infilename).absFilename();
+	infilename = makeAbsPath(infilename).absFileName();
 
 	string outfilename;
-	if (argc > 2) {
+	if (roundtrip) {
+		if (argc > 2) {
+			// Do not allow a user supplied output filename
+			// (otherwise it could easily happen that LyX would
+			// overwrite the original .tex file)
+			cerr << "Error: output filename must not be given in roundtrip mode."
+			     << endl;
+			return EXIT_FAILURE;
+		}
+		outfilename = changeExtension(infilename, ".lyx.lyx");
+	} else if (argc > 2) {
 		outfilename = internal_path(os::utf8_argv(2));
 		if (outfilename != "-")
-			outfilename = makeAbsPath(outfilename).absFilename();
+			outfilename = makeAbsPath(outfilename).absFileName();
 	} else
 		outfilename = changeExtension(infilename, ".lyx");
 
@@ -538,7 +809,7 @@ int main(int argc, char * argv[])
 	FileName const system_syntaxfile = libFileSearch("", "syntax.default");
 	if (system_syntaxfile.empty()) {
 		cerr << "Error: Could not find syntax file \"syntax.default\"." << endl;
-		exit(1);
+		return EXIT_FAILURE;
 	}
 	read_syntaxfile(system_syntaxfile);
 	if (!syntaxfile.empty())
@@ -549,32 +820,37 @@ int main(int argc, char * argv[])
 	if (symbols_path.empty()) {
 		cerr << "Error: Could not find file \"unicodesymbols\"." 
 		     << endl;
-		exit(1);
+		return EXIT_FAILURE;
 	}
 	FileName const enc_path = libFileSearch(string(), "encodings");
 	if (enc_path.empty()) {
 		cerr << "Error: Could not find file \"encodings\"." 
 		     << endl;
-		exit(1);
+		return EXIT_FAILURE;
 	}
 	encodings.read(enc_path, symbols_path);
 	if (!default_encoding.empty() && !encodings.fromLaTeXName(default_encoding))
 		error_message("Unknown LaTeX encoding `" + default_encoding + "'");
 
+	// Load the layouts
+	LayoutFileList::get().read();
+	//...and the modules
+	theModuleList.read();
+
 	// The real work now.
 	masterFilePath = onlyPath(infilename);
 	parentFilePath = masterFilePath;
 	if (outfilename == "-") {
 		if (tex2lyx(FileName(infilename), cout, default_encoding))
 			return EXIT_SUCCESS;
-		else
-			return EXIT_FAILURE;
+	} else if (roundtrip) {
+		if (tex2tex(infilename, FileName(outfilename), default_encoding))
+			return EXIT_SUCCESS;
 	} else {
 		if (tex2lyx(infilename, FileName(outfilename), default_encoding))
 			return EXIT_SUCCESS;
-		else
-			return EXIT_FAILURE;
 	}
+	return EXIT_FAILURE;
 }
 
 // }])
